message "Creating lib CHKL, Please wait .."
createlib "chkl"
release procs wproc
 proc wproc(x)
    execute "writelib \"chkl\" " + x
    + " release procs " + x
 endproc


;================================================== Chkl_ChkList_Dlg
proc Chkl_ChkList_Dlg
; Display a dialog box which contains a (dynarray) picklist. Allow
; user to check selected entries using the spacebar or the mouse.
;
; Return Value:
; Number of checklist entries selected.
(
    check.list    ; {I} The array or dynarray to be displayed in
                  ;     the dialog picklist.
                  ;     If it is an array, then the indexes must
                  ;     start at 1 and end at arraysize. (No gaps.)
                  ;     The dialog box looks funny if there are
                  ;     fewer than 3 entries in the picklist.
                  ;     The contents are NOT modified.
  , checked.list  ; {I,O} Same type (array or dynarray) as 
                  ;       check.list.
                  ;       Entries in this list correspond to
                  ;       entries in check.list (i.e., they have
                  ;       the same indexes).
                  ;       An element's value is set to TRUE if the
                  ;       corresponding element in check.list is
                  ;       selected. Otherwise, it is set to FALSE.
                  ;       This array or dynarray must be allocated 
                  ;       by the caller but it need not have any
                  ;       entries.
                  ;       The caller may set the initial selections
                  ;       by setting entries to true or false.
  , dlg.title     ; {I} Title for dialog box.
  , check.list.lbl; {I} Label for picklist.
                  ;     The label is displayed above the picklist
                  ;     in the dialog box.
  , check.yes     ; {I} The string used to indicate that an item
                  ;     is selected.
                  ;     If blank, then "  " is used. This is:
                  ;     spaces( 2 ) + chr( 251 ).
  , check.no      ; {I} The string used to indicate that an item
                  ;     is deselected.
                  ;     If blank, then "   " is used. This is:
                  ;     spaces( 3 ).
  , buttons.at.bot; {I} If TRUE, then the pushbuttons are
                  ;     displayed at the bottom of the dialog box.
                  ;     Buttons are normally displayed to the right
                  ;     of the picklist. Buttons are automatically
                  ;     moved to the bottom of the box if the
                  ;     picklist width exceeds (approx) 60 chars.
  , help.proc     ; {I} Name of proc to EXECPROC if the user asks
                  ;     help.
                  ;     If blank, generic help is provided.
)
private x, y, z
    , butval
    , scr.hgt          ; screen height
    , scr.wid          ; screen width
    , txtwid           ; width of text in picklist
                       ; (does not include checkmarks).
    , chkwid           ; width of checkmarks
    , work.list        ; array or dynarray for picklist
    , work.ndx         ; current index in work.list
    , dlg.row          ; origin row of dialog box
    , dlg.col          ; origin col of dialog box
    , dlg.hgt          ; height of dialog box
    , dlg.wid          ; width of dialog box
    , dlg.cohd         ; Overhead (in cols) for dialog
    , dlg.rohd         ; Overhead (in rows) for dialog
    , pick.row         ; origin row of picklist in dialog box
    , pick.col         ; origin col of picklist in dialog box
    , pick.hgt         ; height of picklist
    , pick.wid         ; width of picklist
    , pick.cohd        ; Overhead (in cols) for picklist
    , pick.rohd        ; Overhead (in rows) for picklist
    , titl.wid         ; width of dialog title
    , but.row          ; origin row of pushbuttons
    , but.col          ; origin col of pushbuttons
    , but.bot          ; true if pushbuttons are at bottom of box
    , chklist.fmtx     ; format string for picklist text
    , chklist.dyn      ; TRUE if picklist is a dynarray
                       ; FALSE if it is an array
    , chklist.search.count  ; Counter for checklist incremental search.
    , chklist.search.string ; String for checklist incremental search.

chklist.dyn = type( check.list ) = "DY"

; Quit right now if the checklist is empty
if chklist.dyn then
    if dynarraysize( check.list ) = 0 then
        return 0
    endif
else
    if arraysize( check.list ) = 0 then
        return 0
    endif
endif

; Set default help
if isblank( help.proc ) then
    help.proc = "Chkl_ChkList_Dlg_Help"
endif

sysinfo to x
scr.hgt = x[ "screenheight" ]
scr.wid = x[ "screenwidth" ]
release vars x

; Allocate row/col vars for 5 Buttons:
;   Ok, All, None, Cancel, Help
array but.row[ 5 ]
array but.col[ arraysize( but.row ) ]

; Set col overhead for picklist. The picklist has a 1-col margin
; to the left and right. An extra col is saved on the right for the
; scrollbar.
pick.cohd = 3
; Set row overhead for picklist. The picklist has a 1-row margin
; (above the label), a 1-row label, and a 1-row margin below the
; picklist.
pick.rohd = 3

; Set col overhead for dialog. The dialog has a frame (left and right)
; and a shadow (on the right).
dlg.cohd = 3
; Set row overhead for dialog. The dialog has a frame (top and bottom)
; and a shadow (bottom). The pulldown menu area and the status bar are
; also included in the overhead.
dlg.rohd = 5

; The dialog title width is len( title ) + 10 bytes but it can't
; exceed screen width minus 1 (dialog shadow).
titl.wid = min( len( dlg.title) + 10, scr.wid - 1 )

but.bot = false               ; default button pos is to the right
chklist.search.count = 0      ; for incremental search
chklist.search.string  = ""   ; for incremental search

; Set default checkmarks if necessary.
if isblank( check.yes ) then
    check.yes = spaces( 2 ) + chr( 251 )  ; 
    check.no = ""
endif

; Set checkmark width
chkwid = max( len( check.no ), len( check.yes ) )

; Set default label for picklist
if isblank( check.list.lbl ) then
    check.list.lbl = "List"
endif

; The label cannot exceed screen width minus col overhead values.
x = scr.wid - dlg.cohd - pick.cohd
if len( check.list.lbl ) > x then
    check.list.lbl = substr( check.list.lbl, 1, x )
endif

; Fill work.list and calculate txtwid
if chklist.dyn then
    dynarray work.list[]
    txtwid = 1   ; make sure it's at least 1
    foreach x in check.list
        work.list[ x ] = check.list[ x ]
        txtwid = max( txtwid, len( work.list[ x ] ) )
        if not isassigned( checked.list[ x ] ) then 
            checked.list[ x ] = false
        endif
    endforeach
else
    array work.list[ arraysize( check.list ) ]
    txtwid = 1   ; make sure it's at least 1
    for x from 1 to arraysize( check.list )
        work.list[ x ] = check.list[ x ]
        txtwid = max( txtwid, len( work.list[ x ] ) )
        if not isassigned( checked.list[ x ] ) then 
            checked.list[ x ] = false
        endif
    endfor
endif

; The text cannot exceed screen width - col overhead values -
; check mark length.
x = scr.wid - dlg.cohd - pick.cohd - chkwid
txtwid = min( txtwid, x )

; Append check marks to work.list
chklist.fmtx = "W" + strval( txtwid )
if chklist.dyn then
    foreach x in work.list
        work.list[ x ] = format( chklist.fmtx, work.list[ x ] )
            + iif( checked.list[ x ], check.yes, check.no )
    endforeach
    work.ndx = ""   ; force initial position to top of list
else
    for x from 1 to arraysize( work.list )
        work.list[ x ] = format( chklist.fmtx, work.list[ x ] )
            + iif( checked.list[ x ], check.yes, check.no )
    endfor
    work.ndx = 1   ; force initial position to top of list
endif

; Calculate the width of the picklist.
pick.wid = txtwid + chkwid + pick.cohd

; We want the picklist to be at least as wide as the label.
; The label is indented 1 col on the left and right (so it
; aligns with picklist data).
x = len( check.list.lbl ) + pick.cohd
pick.wid = max( pick.wid, x )

; If the picklist width + dialog col overhead + 3 (left/right
; margins in the dialog box and a col between the scrollbar and
; the buttons) + 10 (button width) exceeds the screen width, 
; then force the buttons to be at the bottom of the dialog box.
if pick.wid + dlg.cohd + 13 > scr.wid then
    but.bot = true
endif

; The caller may force buttons to be at the bottom
but.bot = iif( buttons.at.bot, true, but.bot )    

; Calculate remaining dialog box dimensions/parameters.

if chklist.dyn then
    pick.hgt = max( 3, dynarraysize( work.list ) )  ; at least 3
else
    pick.hgt = max( 3, arraysize( work.list ) )  ; at least 3
endif
pick.row = 2

if but.bot then
    if pick.wid < 30 then
        ; If the picklist width is less than 30 ( the width of 3
        ; buttons), then the width of the dialog box will be set
        ; 36 and the buttons will be displayed in two rows across
        ; the bottom of the screen.
        ; The dialog must be wide enough to hold the title.
        dlg.wid = max( 36, titl.wid )

        ; Center the picklist
        pick.col = int( ( dlg.wid - pick.wid - 2 ) / 2 )

        ; The max picklist height is screen height - dialog
        ; row overhead - picklist row overhead - 4 (two rows
        ; of buttons).
        x = scr.hgt - dlg.rohd - pick.rohd - 4
        pick.hgt = min( pick.hgt, x )
        but.row[ 1 ] = pick.row + pick.hgt + 1
        but.row[ 2 ] = but.row[ 1 ]
        but.row[ 3 ] = but.row[ 1 ]
        but.row[ 4 ] = but.row[ 1 ] + 2
        but.row[ 5 ] = but.row[ 1 ] + 2
        but.col[ 1 ] = int( ( dlg.wid - 30 - 2 ) / 2 )
        but.col[ 2 ] = but.col[ 1 ] + 10
        but.col[ 3 ] = but.col[ 2 ] + 10
        but.col[ 4 ] = int( ( dlg.wid - 20 - 2 ) / 2 )
        but.col[ 5 ] = but.col[ 4 ] + 10

        ; Dialog height is picklist height + picklist row
        ; overhead + 4 (buttons) + 2 (dialog frame).
        dlg.hgt = pick.hgt + pick.rohd + 6
    else
        ; Initialize the dialog width to the picklist width + 2
        ; (for the dialog frame).
        ; The dialog must be wide enough to hold the title.
        dlg.wid = max( pick.wid + 2, titl.wid )

        ; If we have room, it would be nice to put a 1-col margin
        ; to the left and right of the picklist. We can do this
        ; if the dialog width + 2 <= screen width - 1 (dialog
        ; shadow).
        if dlg.wid + 2 <= scr.wid - 1 then
            dlg.wid = dlg.wid + 2
        endif

        ; The dialog box must be at least 56 (the width of 5 
        ; buttons + dialog box frame + 2 col margin to left and
        ; right of buttons).
        dlg.wid = max( dlg.wid, 56 )

        ; Center the picklist
        pick.col = int( ( dlg.wid - pick.wid - 2 ) / 2 )

        ; The max picklist height is screen height - dialog
        ; row overhead - picklist row overhead - 2 (a row of
        ; buttons).
        x = scr.hgt - dlg.rohd - pick.rohd - 2
        pick.hgt = min( pick.hgt, x )

        ; Buttons go one row below the picklist
        but.row[ 1 ] = pick.row + pick.hgt + 1

        ; Center the buttons
        but.col[ 1 ] = int( ( dlg.wid - 50 - 2 ) / 2 )

        for x from 2 to 5
            but.row[ x ] = but.row[ x - 1 ]
            but.col[ x ] = but.col[ x - 1 ] + 10
        endfor

        ; Dialog height is picklist height + picklist row
        ; overhead + 2 (buttons) + 2 (dialog frame).
        dlg.hgt = pick.hgt + pick.rohd + 4
    endif
else
    ; The buttons are to the right of the dialog box.

    ; Initialize the dialog width to the picklist width + 10
    ; (button width) + 2 (for the dialog frame).
    ; The dialog must be wide enough to hold the title.
    dlg.wid = max( pick.wid + 12, titl.wid )

    ; If we have room, it would be nice to put a 1-col margin
    ; between the picklist scrollbar and the buttons and a 1-col
    ; margin at the left and right sides of the dialog box.
    ; The dialog width cannot exceed the screen width - 1 (for
    ; the dialog box shadow).
    dlg.wid = min( dlg.wid + 3, scr.wid - 1 )

    ; Distribute blank cols evenly between the left and right
    ; sides of the dialog box and between the picklist and the
    ; buttons.
    x = dlg.wid - 2       ; frame
    x = x - pick.wid      ; picklist
    x = x - 10            ; button width
    pick.col = int( x / 3 )
    but.col[ 1 ] = x - ( 2 * pick.col ) + pick.wid

    ; The max picklist height is screen height - dialog
    ; row overhead - picklist row overhead.
    x = scr.hgt - dlg.rohd - pick.rohd
    pick.hgt = min( pick.hgt, x )

    ; The first button Buttons goes on the same row as the
    ; picklist.
    but.row[ 1 ] = pick.row

    for x from 2 to 5
        but.row[ x ] = but.row[ x - 1 ] + 2
        but.col[ x ] = but.col[ x - 1 ]
    endfor

    ; Dialog height is picklist height + picklist row
    ; overhead + 2 (dialog frame)
    dlg.hgt = pick.hgt + pick.rohd + 2

    ; The dialog box height must be at least 10 (height of 5
    ; buttons) + pick.row (first button is in the same row as
    ; the picklist) + 2 (for dialog frame).
    dlg.hgt = max( dlg.hgt, 12 + pick.row )
endif

; Center the dialog box on the screen
dlg.col = int( ( scr.wid - dlg.wid ) / 2 )  ; bias left
dlg.row = int( ( scr.hgt - dlg.hgt ) / 2 )  ; bias up

butval = "cancel"

if chklist.dyn then
    Chkl_ChkList_Dlg_DynArray()
    z = 0
    if lower( butval ) <> "cancel" then
        foreach x in work.list
            if checked.list[ x ] then
                z = z + 1
            endif
        endforeach
    endif
    return z
else
    Chkl_ChkList_Dlg_Array()
    z = 0
    if lower( butval ) <> "cancel" then
        for x from 1 to arraysize( work.list )
            if checked.list[ x ] then
                z = z + 1
            endif
        endfor
    endif
    return z
endif

endproc
wproc( "Chkl_ChkList_Dlg" )

;================================================== Chkl_ChkList_Dlg_DynArray
proc Chkl_ChkList_Dlg_DynArray
; Displays the dialog box for a checklist whose picklist
; contains a dynarray.
;
; Return Value:
; None.
; All vars are defined in caller.
(
)
SHOWDIALOG
    dlg.title
    PROC "Chkl_ChkList_DlgProc"
        TRIGGER "ALL"
        MOUSE "ALL" 
        KEY "ALL" 
        MESSAGE "ALL"
;;        IDLE
    @ dlg.row, dlg.col
    HEIGHT dlg.hgt
    WIDTH dlg.wid

    LABEL
        @ pick.row - 1, pick.col
        check.list.lbl
        FOR "work.ndx"

    PICKDYNARRAY
        @ pick.row, pick.col
        HEIGHT  pick.hgt
        WIDTH   pick.wid - 1  ; pick.wid includes scroll bar
        COLUMNS 1
        work.list
        TAG "work.ndx"
        TO work.ndx

    PUSHBUTTON
        @ but.row[ 1 ], but.col[ 1 ]
        WIDTH 10
        "~O~k"
        OK
        DEFAULT
        VALUE "ok"
        TAG "ok"
        TO butval

    PUSHBUTTON
        @ but.row[ 2 ], but.col[ 2 ]
        WIDTH 10
        "~A~ll"
        OK
        VALUE "all"
        TAG "all"
        TO butval

    PUSHBUTTON
        @ but.row[ 3 ], but.col[ 3 ]
        WIDTH 10
        "~N~one"
        OK
        VALUE "none"
        TAG "none"
        TO butval

    PUSHBUTTON
        @ but.row[ 4 ], but.col[ 4 ]
        WIDTH 10
        "Cancel"
        Cancel
        VALUE "cancel"
        TAG "cancel"
        TO butval

    PUSHBUTTON
        @ but.row[ 5 ], but.col[ 5 ]
        WIDTH 10
        "~H~elp"
        OK
        VALUE "help"
        TAG "help"
        TO butval

ENDDIALOG

endproc
wproc( "Chkl_ChkList_Dlg_DynArray" )

;================================================== Chkl_ChkList_Dlg_Array
proc Chkl_ChkList_Dlg_Array
; Displays the dialog box for a checklist whose picklist
; contains an array.
;
; Return Value:
; None.
; All vars are defined in caller.
(
)
SHOWDIALOG
    dlg.title
    PROC "Chkl_ChkList_DlgProc"
        TRIGGER "ALL"
        MOUSE "ALL" 
        KEY "ALL" 
        MESSAGE "ALL"
;;        IDLE
    @ dlg.row, dlg.col
    HEIGHT dlg.hgt
    WIDTH dlg.wid

    LABEL
        @ pick.row - 1, pick.col
        check.list.lbl
        FOR "work.ndx"

    PICKARRAY
        @ pick.row, pick.col
        HEIGHT  pick.hgt
        WIDTH   pick.wid - 1  ; pick.wid includes scroll bar
        COLUMNS 1
        work.list
        TAG "work.ndx"
        TO work.ndx

    PUSHBUTTON
        @ but.row[ 1 ], but.col[ 1 ]
        WIDTH 10
        "~O~k"
        OK
        VALUE "ok"
        TAG "ok"
        TO butval

    PUSHBUTTON
        @ but.row[ 2 ], but.col[ 2 ]
        WIDTH 10
        "~A~ll"
        OK
        VALUE "all"
        TAG "all"
        TO butval

    PUSHBUTTON
        @ but.row[ 3 ], but.col[ 3 ]
        WIDTH 10
        "~N~one"
        OK
        VALUE "none"
        TAG "none"
        TO butval

    PUSHBUTTON
        @ but.row[ 4 ], but.col[ 4 ]
        WIDTH 10
        "Cancel"
        Cancel
        VALUE "cancel"
        TAG "cancel"
        TO butval

    PUSHBUTTON
        @ but.row[ 5 ], but.col[ 5 ]
        WIDTH 10
        "~H~elp"
        OK
        VALUE "help"
        TAG "help"
        TO butval

ENDDIALOG

endproc
wproc( "Chkl_ChkList_Dlg_Array" )

;============================================== Chkl_ChkList_Dlg_Help
proc Chkl_ChkList_Dlg_Help
; Generic help screen for the checklist dialog proc.
;
; Return Value:
; None.
(
)
private x, y, z
	, orig.win
	, orig.canv
	, msgw
	, msgx
	, num.lines
	, widest.line
	, msg.hgt
	, msg.wid
	, msg.row
	, msg.col
	, msg.winhand
	, msg.can.wid
	, msg.can.hgt
	, scroll.col
	, scroll.row
	, win.attr

orig.win = getwindow()
orig.canv = getcanvas()

fileread sdir() + "chkldlg.txt" to msgx
dynarray msgw[]
msgw[ "string" ] = msgx
release vars msgx
msgw[ "width" ] = 76
Demo_Word_Wrap( msgw )

; Calc window sizes and locations. Pretty dry stuff.

num.lines = min( msgw[ "lines" ], 200 )
widest.line = msgw[ "longest" ]  ; won't exceed msgw[ "width" ]

msg.can.wid = 78
msg.can.hgt = num.lines
msg.can.hgt = min( msg.can.hgt, 200 )

msg.wid = 80
msg.hgt = 24

msg.row = 0
msg.col = 0

; Create the floating message window

; By the way - this demo assumes a screen size of 24x80.
; We could use sysinfo to get the actual size but this
; is only a demo - not a real application.

dynarray x[]
x[ "canclose" ] = false
x[ "canmaximize" ] = false
x[ "canmove" ] = false
x[ "canresize" ] = false
x[ "canvasheight"] = msg.can.hgt
x[ "canvaswidth" ] = msg.can.wid
x[ "floating" ] = true
x[ "hasframe" ] = true
x[ "hasshadow" ] = false
x[ "height" ] = msg.hgt
x[ "maximized" ] = true
x[ "origincol" ] = msg.col
x[ "originrow" ] = msg.row
x[ "width" ] = msg.wid

window create attributes x to msg.winhand
setcanvas msg.winhand
release vars x

dynarray x[]
x[ 0 ] = syscolor( 1032 )  ; use active color
x[ 1 ] = syscolor( 1032 )
x[ 2 ] = syscolor( 1032 )
x[ 3 ] = syscolor( 1034 )
x[ 4 ] = syscolor( 1035 )

window setcolors msg.winhand from x

; Put the message in the window and paint the entire window
; using the dialog text color.

for x from 1 to num.lines
	@ x - 1, 1 ?? msgw[ x ]
endfor

paintcanvas attribute syscolor( 1036 ) all

; Wait for the user to press Esc.

echo normal
prompt "Press Esc to Exit Help"

while true
	getevent key "all" mouse "all" to evt
	switch
	case evt[ "type" ] = "KEY" :
		if evt[ "keycode" ] = 27 then
			quitloop
		endif
		;
		;   Scrolling keys
		;
		scroll.row = 0
		scroll.col = 0

		switch
		case evt[ "keycode" ] = -71 :  ; Home
			scroll.col = -1000         ; Scroll to left of canvas

		case evt[ "keycode" ] = -72 :  ; Up
			scroll.row = -1

		case evt[ "keycode" ] = -73 :  ; PgUp
			scroll.row = -msg.hgt - 1  ; 1 less than height

		case evt[ "keycode" ] = -75 :  ; Left
			scroll.col = -1

		case evt[ "keycode" ] = -77 :  ; Right
			scroll.col = 1

		case evt[ "keycode" ] = -79 :  ; End
			scroll.col = 1000          ; Scroll to right end of canvas

		case evt[ "keycode" ] = -80 :  ; Down
			scroll.row = 1

		case evt[ "keycode" ] = -81 :  ; PgDn
			scroll.row = msg.hgt - 3   ; 1 less than height

		case evt[ "keycode" ] = -115 : ; Ctrl-Left
			scroll.col = -msg.wid - 1  ; 1 less than width

		case evt[ "keycode" ] = -116 : ; Ctrl-Right
			scroll.col = msg.wid - 3   ; 1 less than width

		case evt[ "keycode" ] = -117 : ; Ctrl-End
			scroll.row = 1000          ; Scroll to bottom of canvas
			scroll.col = 1000          ; Scroll to right end of canvas

		case evt[ "keycode" ] = -118 : ; Ctrl-PgDn
			scroll.row = 1000          ; Scroll to bottom of canvas

		case evt[ "keycode" ] = -119 : ; Ctrl-Home
			scroll.row = -1000         ; Scroll to top of canvas
			scroll.col = -1000         ; Scroll to left end of canvas

		case evt[ "keycode" ] = -132 : ; Ctrl-PgUp
			scroll.row = -1000         ; Scroll to top of canvas

		otherwise :
			beep

		endswitch

		window getattributes msg.winhand to win.attr
		scroll.row = scroll.row + win.attr[ "scrollrow" ]
		scroll.row = max( scroll.row, 0 )
		scroll.row = min( scroll.row
			, win.attr[ "canvasheight" ] - msg.hgt )
		scroll.row = max( scroll.row, 0 )
		scroll.col = scroll.col + win.attr[ "scrollcol" ]
		scroll.col = max( scroll.col, 0 )
		scroll.col = min( scroll.col
			, win.attr[ "canvaswidth" ] - msg.wid )
		scroll.col = max( scroll.col, 0 )
		window scroll msg.winhand to scroll.row, scroll.col

	case evt[ "type" ] = "MOUSE" :
		localizeevent evt
		if evt[ "window" ] = msg.winhand then
			globalizeevent evt
			execevent evt
		endif
	endswitch
endwhile

; Kill the floating window
	
window select msg.winhand
window close

if iswindow( orig.canv ) then
	setcanvas orig.canv
else
	setcanvas default
endif

if iswindow( orig.win ) then
	window select orig.win
endif

prompt ""
echo normal echo off

endproc
wproc( "Chkl_ChkList_Dlg_Help" )

;================================================== Chkl_ChkList_DlgProc
proc Chkl_ChkList_DlgProc
; Dialog proc for Chkl_ChkList_Dlg.
;
; Return Value:
; True if the event is to be processed.
; False if the event is to be ignored.
(
    dlg.trig     ; "Event" or one of the following triggers:
                 ; "Update", "Depart", "Select"
                 ; "Accept", "Cancel", "Arrive"
                 ; "Open", "Close"
  , dlg.tag      ; The tag for the active control element
  , dlg.evt      ; Dynarray for event.
                 ; If "Update", value of active control element.
                 ; If "Depart", tag that you will navigate to.
                 ; Null string if any other trigger.
  , dlg.cb.lbl   ; Null string if event.
                 ; If "Update" trigger, label string of checkbox or
                 ; null string.
                 ; Null string if any other trigger.
)
private x
switch
case dlg.trig = "EVENT" :
    switch
    case dlg.evt[ "type" ] = "IDLE"    : return false
    case dlg.evt[ "type" ] = "KEY"     : return Chkl_ChkList_DlgEvt_Keys()
    case dlg.evt[ "type" ] = "MOUSE"   : return true
    case dlg.evt[ "type" ] = "MESSAGE" : return true
    endswitch
case dlg.trig = "UPDATE" : return true
case dlg.trig = "DEPART" : return true
case dlg.trig = "SELECT" : return Chkl_ChkList_DlgTrg_Select()
case dlg.trig = "ACCEPT" : return Chkl_ChkList_DlgTrg_Accept()
case dlg.trig = "ARRIVE" : return true
case dlg.trig = "OPEN"   : return Chkl_ChkList_DlgTrg_Open()
case dlg.trig = "CANCEL" : return true
case dlg.trig = "CLOSE"  : return true
endswitch
return true

endproc
wproc( "Chkl_ChkList_DlgProc" )

;============================================== Chkl_ChkList_DlgEvt_Keys
proc Chkl_ChkList_DlgEvt_Keys
; Handles KEY events for the checklist dialog.
;
; Return Value:
; True or false.
(
)
private x, y, z
    , sp

; Increment counter for checklist incremental search
chklist.search.count = chklist.search.count + 1

; Treat F2 as Alt-O (for the OK button)
if dlg.evt[ "keycode" ] = asc( "F2" ) then
    release vars dlg.evt[ "scancode" ]
    dlg.evt[ "keycode" ] = -24
    return true
endif

; It's not really necessary, but we'll kill the Ctrl-O key.
; We allow Alt-O because it is tied to the Ok button for the dialog.
if dlg.evt[ "keycode" ] = 15 then
    beep
    return false
endif

; Show help if the user presses F1
if dlg.evt[ "keycode" ] = asc( "F1" ) then
    execproc help.proc
    return false
endif

; Treat the enter key as the spacebar (select) while in the checklist
if lower( dlg.tag ) = "work.ndx" and
   dlg.evt[ "keycode" ] = asc( "Enter" ) then
    release vars dlg.evt[ "scancode" ]
    dlg.evt[ "keycode" ] = 32  ; spacebar
    return true
endif

; We only care about ASCII keystrokes < 127 in the checklist field.
if lower( dlg.tag ) = "work.ndx  - disable search" and
   dlg.evt[ "keycode" ] > 32 and
   dlg.evt[ "keycode" ] < 127 then
    z = upper( chr( dlg.evt[ "keycode" ] ) )

    ; If the number of keystrokes since the last incremental
    ; search key is greater than 1, then reset the search field.
    if chklist.search.count > 1 then
        chklist.search.string = z
    else
        chklist.search.string = chklist.search.string + z
    endif
    ; If the search string is 1 byte long, start the search at the
    ; top of the list. Otherwise, start at current entry.
    if len( chklist.search.string ) = 1 then
        sp = 1
    else
        sp = work.ndx - 1
    endif
    chklist.search.count = 0        ; reset counter
    z = chklist.search.string
    for x from sp to dynarraysize( dir.list )
        if z = substr( dir.list[ x ], 1, len(z) ) then
            work.ndx = x + 1
            refreshcontrol "work.ndx"
            return false
        endif
    endfor
    ; Search failed - Strip last character
    if len( z ) < 2 then
        chklist.search.string = ""
    else
        chklist.search.string = substr( z, 1, len( z ) - 1 )
    endif
    return true
endif

return true

endproc
wproc( "Chkl_ChkList_DlgEvt_Keys" )

;============================================== Chkl_ChkList_DlgTrg_Accept
proc Chkl_ChkList_DlgTrg_Accept
; Handles ACCEPT trigger for the checklist dialog.
;
; Return Value:
; True or false.
(
)
private x

switch
case lower( dlg.tag ) = "ok" :
    ; If user pushed Ok, then allow it unconditionally.
    return true

case lower( dlg.tag ) = "help" :
    ; If user pushed HELP, then show the help and cancel the accept.
    execproc help.proc
    selectcontrol "work.ndx"
    return false

case lower( dlg.tag ) = "all" :
    ; If user pushed All, then select all elements and cancel the accept.
    Chkl_ChkList_Dlg_All_Yes()
    refreshcontrol "work.ndx"
    selectcontrol "work.ndx"
    return false

case lower( dlg.tag ) = "none" :
    ; If user pushed None, then deselect all elements and cancel the accept.
    Chkl_ChkList_Dlg_All_No()
    refreshcontrol "work.ndx"
    selectcontrol "work.ndx"
    return false

endswitch

return true

endproc
wproc( "Chkl_ChkList_DlgTrg_ACCEPT" )

;============================================== Chkl_ChkList_DlgTrg_Select
proc Chkl_ChkList_DlgTrg_Select
; Handles SELECT trigger for the checklist dialog.
;
; Return Value:
; True or false.
(
)
private x

switch
case lower( dlg.tag ) = "work.ndx" :
    ; Toggle the select for the checklist picklist
    checked.list[ work.ndx ] = not checked.list[ work.ndx ]
    work.list[ work.ndx ] = 
        format( chklist.fmtx, work.list[ work.ndx ] )
        + iif( checked.list[ work.ndx ], check.yes, check.no )
    refreshcontrol "work.ndx"
    return true
endswitch

return true

endproc
wproc( "Chkl_ChkList_DlgTrg_Select" )

;============================================== Chkl_ChkList_DlgTrg_Open
proc Chkl_ChkList_DlgTrg_Open
; Handles OPEN trigger for the checklist dialog.
;
; Return Value:
; True or false.
(
)
private x

return true

endproc
wproc( "Chkl_ChkList_DlgTrg_Open" )

;============================================== Chkl_ChkList_Dlg_All_Yes
proc Chkl_ChkList_Dlg_All_Yes
; Set all checklist entries to TRUE (selected)
;
; Return Value:
; None.
; All entries in work.list are marked as selected.
; Corresponding entries in checked.list are set to TRUE.
(
)
private x

if chklist.dyn then
    foreach x in work.list
        checked.list[ x ] = true
        work.list[ x ] = format( chklist.fmtx, work.list[ x ] ) + check.yes
    endforeach
else
    for x from 1 to arraysize( work.list )
        checked.list[ x ] = true
        work.list[ x ] = format( chklist.fmtx, work.list[ x ] ) + check.yes
    endfor
endif

endproc
wproc( "Chkl_ChkList_Dlg_All_Yes" )

;============================================== Chkl_ChkList_Dlg_All_No
proc Chkl_ChkList_Dlg_All_No
; Set all checklist entries to FALSE (deselected)
;
; Return Value:
; None.
; All entries in work.list are marked as deselected.
; Corresponding entries in checked.list are set to FALSE.
(
)
private x

if chklist.dyn then
    foreach x in work.list
        checked.list[ x ] = false
        work.list[ x ] = format( chklist.fmtx, work.list[ x ] ) + check.no
    endforeach
else
    for x from 1 to arraysize( work.list )
        checked.list[ x ] = false
        work.list[ x ] = format( chklist.fmtx, work.list[ x ] ) + check.no
    endfor
endif

endproc
wproc( "Chkl_ChkList_Dlg_All_No" )

